// AECioctl.h    Include file for AecPciTC Driver
//
// Define the IOCTL codes we will use.  The IOCTL code contains a command
// identifier, plus other information about the device, the type of access
// with which the file must have been opened, and the type of buffering.
//

// Device type           -- in the "User Defined" range."
#define AECPCI_TYPE 40000

// The IOCTL function codes from 0x800 to 0xFFF are for customer use.

#define IOCTL_AEC_READ_PORT_UCHAR \
    CTL_CODE( AECPCI_TYPE, 0x900, METHOD_BUFFERED, FILE_READ_ACCESS )

#define IOCTL_AEC_READ_PORT_USHORT \
    CTL_CODE( AECPCI_TYPE, 0x901, METHOD_BUFFERED, FILE_READ_ACCESS )

#define IOCTL_AEC_READ_PORT_ULONG \
    CTL_CODE( AECPCI_TYPE, 0x902, METHOD_BUFFERED, FILE_READ_ACCESS )

#define IOCTL_AEC_WRITE_PORT_UCHAR \
    CTL_CODE(AECPCI_TYPE,  0x910, METHOD_BUFFERED, FILE_WRITE_ACCESS)

#define IOCTL_AEC_WRITE_PORT_USHORT \
    CTL_CODE(AECPCI_TYPE,  0x911, METHOD_BUFFERED, FILE_WRITE_ACCESS)

#define IOCTL_AEC_WRITE_PORT_ULONG \
    CTL_CODE(AECPCI_TYPE,  0x912, METHOD_BUFFERED, FILE_WRITE_ACCESS)

typedef struct  _GENPORT_WRITE_INPUT {
    ULONG   PortNumber;     // Port # to write to
    union   {               // Data to be output to port
        ULONG   LongData;
        USHORT  ShortData;
        UCHAR   CharData;
    };
}   GENPORT_WRITE_INPUT;


#define	BIT0	0x01
#define	BIT1	0x02
#define	BIT2	0x04
#define	BIT3	0x08
#define	BIT4	0x10
#define	BIT5	0x20
#define	BIT6	0x40
#define	BIT7	0x80


// ----------------------------------------------------------------------------
// ------------------------------------------------------------------------------
/*++
// Example of the enum keyword
enum Days               // Declare enum type Days
{
   saturday,            // saturday = 0 by default
   sunday = 0,          // sunday = 0 as well
   monday,              // monday = 1
   tuesday,             // tuesday = 2
   wednesday,           // etc.
   thursday,
   friday
} today;                // Variable today has type Days
--*/

enum AECPCI_ACTION  // ACTION for AECPCI_ACTION_ITEM "action" buffer
{                      // action naming convention:
                        //       data size
                        //     1 - byte  8 bits,   2 - word 16 bits,  4 - long 32 bits
                        //       port or memory mapped I/O from base address of device
                        //                  param1            param2
    NOP_THING,         //  No operation		0                   0	//DONT'T USE
    Read1,              //  input byte			offset				data (in)	
    Read2,              //  input word			offset				data (in)
    Read4,              //  input long			offset				data (in)
    Write1,             //  output byte			offset				data (out)
    Write2,             //  output word 		offset				data (out)
    Write4,             //  output long 		offset				data (out)

	R256=256,		//NOT supported YET! - (fixed offset write/read... operation)
	SND_CMD,		//send command			command(out)		intcode(in)
					//read the NEXT INTCODE - fixed OFFSET read / enable next
	GET_INT,		//get int code			0 (don't care?)		intcode(in) UCHAR
	RTM,			//  input long			0 (don't care?)		data (out)
					//Read the TIME bits at offset 10h (transition tolerent) DWORD
	RUB,			//  input long			0 (don't care?)		data (out)
					//Read the USER bits at offset 14h (transition tolerent) DWORD
	REB,			//  input byte			0 (don't care?)		data (out)
					//Read the EMBEDDED bits at offset 18h (transition tolerent) UCHAR
	NextInt,		//  output byte			0 (don't care)		data (out)
					//Queue an IRP for the ISR/DPC to complete


	LAST			//nothing!
}; 
// ----------------------------------------------------------------------------
// Device type 
#define AEC_TYPE 40010
// The Device IOCTL function codes
        // Process AECPCI command buffer
        //   input/output:  use array of AECPCI_ACTION_ITEMs for input & output
#define IOCTL_AECPCI_PROCESS_BUFFER \
    CTL_CODE( AEC_TYPE, 0xA00, METHOD_BUFFERED, FILE_ANY_ACCESS )
// ----------------------------------------------------------------------------
            // Process buffer item for input to IOCTL_AECPCI_PROCESS_BUFFER
            //                              and IOCTL_AECPCI_WAIT_ON_INTERRUPT
typedef struct _AECPCI_ACTION_ITEM
{
    enum AECPCI_ACTION  action; // action to perform
    ULONG               port;       // port address
    ULONG               value;      // input or output data
} AECPCI_ACTION_ITEM, *PAECPCI_ACTION_ITEM;
// ----------------------------------------------------------------------------

        // AecIOAction - process an input/output buffer
        //     Inputs:  h  - handle to device 
        //                  (returned from OpenDevice())
        //              pBuf - pointer to i/o buffer
        //                  (array of type AECPCI_ACTION_ITEM)
        //              size  - size off i/o buf       
        //     Outputs: pLen  - pointer to variable which contains
        //                  length of returned buffer (DWORD)
        //              returns - error if zero
        //                   (call GetLastError() to error code)                     
        //      notes: pB is both an input & output
#define AecIOAction(hAecDev,pBuf,size,pLen)\
            DeviceIoControl(\
                hAecDev, (DWORD)IOCTL_AECPCI_PROCESS_BUFFER,\
                pBuf, size, pBuf, size, pLen, NULL)


/*++
//		oneb = Read1 (IO_Offset);	
//%%
{
	AECPCI_ACTION_ITEM _AECPCIpp[] =
	{//	command	addr_offset	data
		{R1,	0,	0},
	};
		_AECPCIpp[ 0].port = IO_Offset;
	(void)AecIOAction(hAecDev, _WinRTpp,
				sizeof(_AECPCIpp01), &iWinRTlength);
	oneb = _AECPCIpp[ 0].value;
--*/
