/*++

Copyright (c) 1993  Microsoft Corporation

Module Name:

    Instdrv.c

Abstract:

    A simple Win32 app that installs 
	(a device driver) the AecPciTC.sys driver
	from the A: drive

Environment:

    user mode only

Notes:

    See readme.txt

Revision History:

    06-25-93 : created
--*/



#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAX_LOADSTRING 128


BOOL
InstallDriver(
    IN SC_HANDLE  SchSCManager,
    IN LPCTSTR    DriverName,
    IN LPCTSTR    ServiceExe
    );

BOOL
RemoveDriver(
    IN SC_HANDLE  SchSCManager,
    IN LPCTSTR    DriverName
    );

BOOL
StartDriver(
    IN SC_HANDLE  SchSCManager,
    IN LPCTSTR    DriverName
    );

BOOL
StopDriver(
    IN SC_HANDLE  SchSCManager,
    IN LPCTSTR    DriverName
    );

BOOL
OpenDevice(
    IN LPCTSTR    DriverName
    );



VOID
__cdecl
main(
    IN int   argc,
    IN char *argv[]
    )
/*++

Routine Description:

Arguments:

Return Value:

--*/
{
    SC_HANDLE   schSCManager;

    if (argc != 3)
    {
        char currentDirectory[128];

        printf ("usage: instdrv aecpcitc load\n");	// put driver file and path
        printf ("           to install a kernel-mode device driver, or:\n");
        printf ("       instdrv aecpcitc remove\n");
        printf ("           to remove a kernel-mode device driver\n\n");

//        GetCurrentDirectory (128,
//                             currentDirectory
//                             );
//
//        //printf ("       Example: instdrv simpldrv %s\\obj\\i386\\simpldrv.sys\n",
//        printf ("       Example: instdrv AecPciTC load\n",
//                currentDirectory
//                );

        exit (1);
    }

	//Get handle to SCManager
    schSCManager = OpenSCManager (NULL,                 // machine (NULL == local)
                                  NULL,                 // database (NULL == default)
                                  SC_MANAGER_ALL_ACCESS // access required
                                  );

    if ((_stricmp (argv[1],	"aecpcitc") != 0))			//Return Value Description:
														//   < 0 string1 less than string2 
														//     0 string1 identical to string2 
														//   > 0 string1 greater than string2 
                  
    {	
        printf (" 'aecpcitc' must be second parameter in command line!\n   ( ie: a:\\>instdrv aecpcitc load )\n" );

        exit (1);

	}

    if (!_stricmp (argv[2], "remove" ))
    {

        StopDriver (schSCManager,
                  argv[1]
                  );

        RemoveDriver (schSCManager,
                      argv[1]
                      );
    }
    else
	
    {
        InstallDriver (schSCManager,	//IN SC_HANDLE  SchSCManager,
                       argv[1],			//IN LPCTSTR    DriverName,
                       argv[2]			//IN LPCTSTR    ServiceExe (path\...sys)
                       );

//        StartDriver (schSCManager,
//                     argv[1]
//                     );
//
//        OpenDevice (argv[1]);

    }

    CloseServiceHandle (schSCManager);
}



BOOL
InstallDriver(
    IN SC_HANDLE  SchSCManager,
    IN LPCTSTR    DriverName,	//arg[1]
    IN LPCTSTR    ServiceExe	//arg[2]	path
    )
/*++

Routine Description:

Arguments:

Return Value:

--*/
{
    SC_HANDLE  schService;
    DWORD      err;
	BOOL		bVal;

	LPCTSTR		lpDestination[256];
	LPCTSTR		lpSource = "a:\\AecPciTC.sys";
	LPCTSTR		szPath[MAX_LOADSTRING];	//was TCHAR
    LPCTSTR		currentDirectory[256];	//was TCHAR

    //char currentDirectory[128];

    GetCurrentDirectory (128, (char*)currentDirectory);	//(DWORD len, LPTSTR lpBuf);
	//debug ---->
//	printf ("  Current directory is: \n    %s\n",
//	                currentDirectory
//						);
	// <----- debug
	strcat( (char*)currentDirectory, "\\AecPciTC.sys" );

	//debug ----->
//	printf ("\nSTRCAT-->    %s\nlpSource --> %s\n", currentDirectory, lpSource);
	// <------ debug 



	//==============================================================
	//Copy the driver file to the sytem32\drivers directory of the "windows" dir.

	GetWindowsDirectory((char*)lpDestination, sizeof(lpDestination));
	strcpy( (char*)szPath, (char*)lpDestination);


	strcat( (char*)szPath, "\\system32\\drivers\\aecpcitc.sys" );
	//debug ---->
//	printf ("  Driver destination is: \n    %s\n",
//	                szPath
//						);
	// <----- debug

	//lpDestination = "c:\\AecPciTC.sys";      //works
	//lpDestination = "c:\\windows\\system32\\drivers\\AecPciTC.sys";
	//bVal = CopyFile(lpSource, lpDestination, FALSE); 
//	bVal = CopyFile(lpSource, (char*)szPath, FALSE); 
	bVal = CopyFile((char*)currentDirectory, (char*)szPath, FALSE); 
    if (!bVal)	//if FALSE /"0" error
    {
			err = GetLastError();
			printf ("failure: Copying file AecPciTC.sys to drivers directory (0x%02x)\n", err );
	}

	strcpy( (char*)lpDestination, "\\SystemRoot\\system32\\drivers\\aecpcitc.sys" );	//initialize for use in registry entry
	//==============================================================


    //
    // NOTE: This creates an entry for a standalone driver. If this
    //       is modified for use with a driver that requires a Tag,
    //       Group, and/or Dependencies, it may be necessary to
    //       query the registry for existing driver information
    //       (in order to determine a unique Tag, etc.).
    //

    schService = CreateService (SchSCManager,          // SCManager database
                                DriverName,           // name of service
                                DriverName,           // name to display
                                SERVICE_ALL_ACCESS,    // desired access
                                SERVICE_KERNEL_DRIVER, // service type
                                //SERVICE_DEMAND_START,  // start type
								SERVICE_AUTO_START,
                                SERVICE_ERROR_NORMAL,  // error control type
                                //ServiceExe,            // service's binary (string contains fully 
														//	qualified path to service binary)
                                (char*)lpDestination, //szPath,					// service's binary (string contains fully 
														//	qualified path to service binary)
                                //NULL,            // service's binary (string contains fully 
														//	qualified path to service binary)
                                NULL,                  // no load ordering group
                                NULL,                  // no tag identifier
                                NULL,                  // no dependencies
                                NULL,                  // LocalSystem account
                                NULL                   // no password
                                );
/*++

  --*/


    if (schService == NULL)
    {
        err = GetLastError();

        if (err == ERROR_SERVICE_EXISTS)
        {
            //
            // A common cause of failure (easier to read than an error code)
            //

            printf ("failure: CreateService, ERROR_SERVICE_EXISTS\n");
        }
        else
        {
            printf ("failure: CreateService (0x%02x)\n",
                    err
                    );
        }

        return FALSE;
    }
    else
    {
        printf ("CreateService SUCCESS\n");
		printf ("You must REBOOT the computer before changes will take effect.\n\n");
    }

    CloseServiceHandle (schService);

    return TRUE;
}

/*++
VOID CreateSampleService() 
{ 
    LPCTSTR lpszBinaryPathName = 
        "%SystemRoot%\\system\\testserv.exe"; 
 
    schService = CreateService( 
        schSCManager,              // SCManager database 
        "Sample_Srv",              // name of service 
        lpszDisplayName,           // service name to display 
        SERVICE_ALL_ACCESS,        // desired access 
        SERVICE_WIN32_OWN_PROCESS, // service type 
        SERVICE_DEMAND_START,      // start type 
        SERVICE_ERROR_NORMAL,      // error control type 
        lpszBinaryPathName,        // service's binary 
        NULL,                      // no load ordering group 
        NULL,                      // no tag identifier 
        NULL,                      // no dependencies 
        NULL,                      // LocalSystem account 
        NULL);                     // no password 


    if (schService == NULL) 
        MyErrorExit("CreateService"); 
    else 
        printf("CreateService SUCCESS.\n"); 
 
    CloseServiceHandle(schService); 
} 

  --*/




BOOL
RemoveDriver(
    IN SC_HANDLE  SchSCManager,
    IN LPCTSTR    DriverName
    )
/*++

Routine Description:

Arguments:

Return Value:

--*/
{
    SC_HANDLE  schService;
    BOOL       ret;

    schService = OpenService (SchSCManager,
                              DriverName,
                              SERVICE_ALL_ACCESS
                              );

    if (schService == NULL)
    {
        printf ("failure: OpenService (0x%02x)\n", GetLastError());
        return FALSE;
    }

    ret = DeleteService (schService);

    if (ret)
    {
        printf ("DeleteService SUCCESS\n");
    }
    else
    {
        printf ("failure: DeleteService (0x%02x)\n",
                GetLastError()
                );
    }

    CloseServiceHandle (schService);

    return ret;
}



BOOL
StartDriver(
    IN SC_HANDLE  SchSCManager,
    IN LPCTSTR    DriverName
    )
{
    SC_HANDLE  schService;
    BOOL       ret;
    DWORD      err;

    schService = OpenService (SchSCManager,
                              DriverName,
                              SERVICE_ALL_ACCESS
                              );

    if (schService == NULL)
    {
        printf ("failure: OpenService (0x%02x)\n", GetLastError());
        return FALSE;
    }

    ret = StartService (schService,    // service identifier
                        0,             // number of arguments
                        NULL           // pointer to arguments
                        );
    if (ret)
    {
        printf ("StartService SUCCESS\n");
    }
    else
    {
        err = GetLastError();

        if (err == ERROR_SERVICE_ALREADY_RUNNING)
        {
            //
            // A common cause of failure (easier to read than an error code)
            //

            printf ("failure: StartService, ERROR_SERVICE_ALREADY_RUNNING\n");
        }
        else
        {
            printf ("failure: StartService (0x%02x)\n",
                    err
                    );
        }
    }

    CloseServiceHandle (schService);

    return ret;
}



BOOL
StopDriver(
    IN SC_HANDLE  SchSCManager,
    IN LPCTSTR    DriverName
    )
{
    SC_HANDLE       schService;
    BOOL            ret;
    SERVICE_STATUS  serviceStatus;

    schService = OpenService (SchSCManager,
                              DriverName,
                              SERVICE_ALL_ACCESS
                              );

    if (schService == NULL)
    {
        printf ("failure: OpenService (0x%02x)\n", GetLastError());
        return FALSE;	//ERROR_INVALID_SERVICE_CONTROL
    }

    ret = ControlService (schService,
                          SERVICE_CONTROL_STOP,
                          &serviceStatus
                          );
    if (ret)
    {
        printf ("ControlService SUCCESS\n");
    }
    else
    {
        printf ("failure: ControlService (0x%02x)\n",
                GetLastError()
                );
    }

    CloseServiceHandle (schService);

    return ret;
}



BOOL
OpenDevice(
    IN LPCTSTR    DriverName
    )
{
    char     completeDeviceName[64] = "";
    LPCTSTR  dosDeviceName = DriverName;
    HANDLE   hDevice;
    BOOL     ret;



    //
    // Create a \\.\XXX device name that CreateFile can use
    //
    // NOTE: We're making an assumption here that the driver
    //       has created a symbolic link using it's own name
    //       (i.e. if the driver has the name "XXX" we assume
    //       that it used IoCreateSymbolicLink to create a
    //       symbolic link "\DosDevices\XXX". Usually, there
    //       is this understanding between related apps/drivers.
    //
    //       An application might also peruse the DEVICEMAP
    //       section of the registry, or use the QueryDosDevice
    //       API to enumerate the existing symbolic links in the
    //       system.
    //

    strcat (completeDeviceName,
            "\\\\.\\"
            );

    strcat (completeDeviceName,
            dosDeviceName
            );

    hDevice = CreateFile (completeDeviceName,
                          GENERIC_READ | GENERIC_WRITE,
                          0,
                          NULL,
                          OPEN_EXISTING,
                          FILE_ATTRIBUTE_NORMAL,
                          NULL
                          );

    if (hDevice == ((HANDLE)-1))
    {
        printf ("Can't get a handle to %s\n",
                completeDeviceName
                );

        ret = FALSE;
    }
    else
    {
        printf ("CreateFile SUCCESS\n");

        CloseHandle (hDevice);

        ret = TRUE;
    }

    return ret;
}
